function [t,y]=odefsrk(fhandle,tspan,yic,par,maxt)
% This routine solves a ordinary differential equation
% using a 4th order Runga-Kutta method.
% 
% [t,y] = odefsrk(fhandle,tspan,yic,par,maxt);
%
% Inputs:
%
% fhandle  = a handle to the function whose output is the time derivative
%            of the system model.  The inputs to this function are time,
%            state, and parameter vales.
% tspan    = a vector whose elements describe at which point in time the
%            solution is sought
% yic      = a vector which describes the initial condition of the system
%            being simulated
% par      = a structure which cointains data or parameters needed to 
%            evaluate the time derivative of the state variables
% maxt     = the maximum allowed time step
% 
% Outputs:
%
% t        = a vector of times at which the state vector has been found
% y        = a matrix wherein each row cointains the state vector at a
%            given time.  Each column is the time history of a particular
%            state
%
% Internal:
%
% Nt       = Number of points at which to calculate the state vector
% Ns       = Number of state variables
% i        = index variable for time
% deltat   = time span between current interval being simulated (s)
% nsteps   = number of integration steps required on interval
% h        = integration time step (s)
% ho2      = half of integration time step (s)
% y1       = state vector at beginning of integration step
% t1       = time at the beginning of the integration step (s)
% k1       = time derivative of state vector at beginning of integration
%            step (s) 
% y2       = 1st est. of state vector in center of integration step 
% k2       = time derivative of y2 at center of integration step
% y3       = 2nd est. of state vector in center of integration step
% k3       = time derivative of y3 at center of integration step
% y4       = estimate of state vector at end of integration step
% k4       = time derivative of state vector at end of integration step
% k        = weighted average value of time derivative of integration
%            step
%
% Written by S.D. Sudhoff 
%   School of Electrical and Computer Engineering
%   1285 Electrical Engineering Building
%   West Lafayette, IN 47907-1285
%   Phone: 765-494-3246
%   Fax: 765-494-0676
%   E-mail: sudhoff@ecn.purdue.edu

% determine number of times at which state vector is recorded
Nt=length(tspan);

% determine the nubmer of states
Ns=length(yic);

% assign the time vector, and initialize y matrix
t=tspan;
y=zeros(Nt,Ns);

% first value row of y matrix is determined in accordance with
% initial condition
y(1,:)=yic;

% iterate one communication interval at a time.  A given communication
% interval may require sevaral time steps
for i=2:Nt;

   % find integration information for this communication interval
   deltat=tspan(i)-tspan(i-1); 
   nsteps=ceil(deltat/maxt);
   h=deltat/nsteps;
   ho2=0.5*h;
   y1=y(i-1,:)';
   t1=tspan(i-1);
   
   % now perform enough integration steps to get through communication
   % inteval
   for j=1:nsteps
       
      % perform Runga-Kutta updata 
      t2=t1+ho2;
      t3=t1+h;
       
      k1=fhandle(t1,y1,par);
      
      y2=y1+ho2*k1;
      k2=fhandle(t2,y2,par);
      
      y3=y1+ho2*k2;
      k3=fhandle(t2,y3,par);
      
      y4=y1+h*k3;
      k4=fhandle(t3,y4,par);
      
      k=(k1+2.0*k2+2.0*k3+k4)/6.0;
      y1=y1+k*h;
      
      t1=t1+h;
           
   end
   
   % fill in the y-matrix
   y(i,:)=y1';
    
end